import cv2

from config import MODEL_PATH, VIDEO_PATH, LABEL_PATH, MODEL_NAME
from vision.ssd.mobilenetv1_ssd import create_mobilenetv1_ssd, create_mobilenetv1_ssd_predictor

train_model = MODEL_PATH / MODEL_NAME
test_videos = VIDEO_PATH.glob('*.avi')

class_names = [name.strip() for name in LABEL_PATH.read_text().split()]
net = create_mobilenetv1_ssd(len(class_names), is_test=True)
net.load(train_model)
predictor = create_mobilenetv1_ssd_predictor(net, nms_method='hard')

count = 0
for video in test_videos:
    count += 1
    cap = cv2.VideoCapture(str(video))
    if count != 1:
        continue

    while True:
        suc, bgr = cap.read()
        if not suc:
            break

        rgb = cv2.cvtColor(bgr, cv2.COLOR_BGR2RGB)
        boxes, labels, probs = predictor.predict(rgb, 5, 0.4)
        for i in range(boxes.size(0)):
            box = boxes[i, :]
            if box[0] <= 0 or box[1] <= 0 or box[3] >= 240:
                continue

            label = f"{class_names[labels[i]]}: {probs[i]:.2f}"
            print(label)

            cv2.rectangle(bgr, (int(box[0]), int(box[1])), (int(box[2]), int(box[3])), (255, 255, 0), 4)
            cv2.putText(bgr, label, (int(box[0]) + 20, int(box[1]) + 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (255, 0, 255), 2)

        cv2.imshow('bgr', bgr)
        cv2.waitKey(1)

    break
